@@ -24,7 +24,8 @@
 #include <stdlib.h>
 #include <guile/gh.h>
 #include <dirent.h>
-#include "gnome.h"
+#include <gnome.h>
+#include <games-frame.h>
 #include "cscmi.h"
 #include "sol.h"
 #include "menu.h"
@@ -33,29 +34,31 @@
 #include "events.h"
 
 static GtkWidget *hint_dlg = NULL;
+
 void show_game_over_dialog() {
   GtkWidget* dialog;
   gchar* message;
 
   if (game_won)
-    message = _("Congratulations\n\nYou Won!!!");
+    message = g_strdup_printf ("<b>%s\n\n%s</b>", _("Congratulations."),
+                                _("You won!!!"));
   else
-    message = _("\nGame Over.\n");
+    message = g_strdup_printf ("<b>%s</b>", _("Game over."));
 
-  /* Should we use the status bar for game over reports when the user
-   * look & feel prefs go for the status bar ? I think not... */
-#if 1
   dialog = gtk_message_dialog_new (GTK_WINDOW (app),
                                    GTK_DIALOG_DESTROY_WITH_PARENT,
-                                   GTK_MESSAGE_QUESTION,
+                                   GTK_MESSAGE_INFO,
                                    GTK_BUTTONS_NONE,
                                    message);
 
+  g_free (message);
+  gtk_label_set_use_markup (GTK_LABEL (GTK_MESSAGE_DIALOG (dialog)->label),
+                            TRUE);
   gtk_dialog_add_buttons (GTK_DIALOG (dialog),
                           _("New Game"), GTK_RESPONSE_ACCEPT,
                           GTK_STOCK_QUIT, GTK_RESPONSE_REJECT,
                           NULL);
-
+  gtk_dialog_set_has_separator (GTK_DIALOG (dialog), FALSE);
   gtk_dialog_set_default_response (GTK_DIALOG (dialog), GTK_RESPONSE_YES);
 
   /* add a stock icon? */ 
@@ -79,15 +82,11 @@ void show_game_over_dialog() {
        gtk_widget_destroy (dialog);
      }
   }
-
-#else
-  gnome_app_question_modal ( GNOME_APP (app), message, random_seed, NULL);
-#endif
 }
 
 gchar *filename = NULL;
 
-void select_rules(GtkTreeSelection *select, gpointer data)
+static void select_rules(GtkTreeSelection *select, gpointer data)
 {
 	GtkTreeModel *model;
         GtkTreeIter iter;
@@ -96,7 +95,7 @@ void select_rules(GtkTreeSelection *select, gpointer data)
 	gtk_tree_model_get(model, &iter, 1, &filename, -1);
 }
 
-void select_game (GtkWidget *app, gint response, GtkWidget* entry)
+static void select_game (GtkWidget *app, gint response, GtkWidget* entry)
 {
   if(response == GTK_RESPONSE_OK) {
     seed = atoi (gtk_entry_get_text (GTK_ENTRY (entry)));
@@ -117,6 +116,7 @@ void show_select_game_dialog()
   GtkTreeViewColumn* column;
   GtkCellRenderer* renderer;
   GtkTreeIter iter;
+  GtkTreeIter selected_iter;
   GtkTreePath * path;
   
   guint i;
@@ -135,7 +135,7 @@ void show_select_game_dialog()
                                           GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
                                           GTK_STOCK_OK, GTK_RESPONSE_OK,
                                           NULL);
-
+    gtk_dialog_set_has_separator (GTK_DIALOG (dialog), FALSE);
     hbox = gtk_hbox_new (FALSE, GNOME_PAD_SMALL);
     seed_entry = gtk_entry_new ();
     label = gtk_label_new(_("Seed"));
@@ -183,22 +183,20 @@ void show_select_game_dialog()
 		      GNOME_PAD_SMALL );
   
     gtk_box_pack_end (GTK_BOX (GTK_DIALOG (dialog)->vbox), 
-		      hbox, FALSE, FALSE, GNOME_PAD_SMALL );
-
-    gtk_widget_show_all (dialog);
+		      hbox, TRUE, TRUE, GNOME_PAD_SMALL );
 
     filename = NULL;
+    selected_iter.stamp = 0;
     
     for(i = 0; i < n_games; i++) {
 	    gchar *text;
-	    gint row;
 	    text = game_file_to_name (game_dents[i]->d_name);
 	    gtk_list_store_append (GTK_LIST_STORE (list), &iter);
             gtk_list_store_set(GTK_LIST_STORE (list), 
                                &iter, 0, text, 1,
                                game_dents[i]->d_name, -1);
             if (g_utf8_collate(text,game_name) == 0) {
-              gtk_tree_selection_select_iter (select, &iter);
+                    memcpy (&selected_iter, &iter, sizeof(GtkTreeIter));
             }
     }
 
@@ -211,6 +209,10 @@ void show_select_game_dialog()
     g_signal_connect (GTK_WIDGET (dialog), "delete_event",
 		                  GTK_SIGNAL_FUNC(gtk_widget_hide), NULL);
 
+    gtk_widget_show_all (dialog);
+
+    if (selected_iter.stamp != 0)
+      gtk_tree_selection_select_iter (select, &selected_iter);    
   }
   
   g_snprintf (buf, sizeof (buf), "%d", seed);
@@ -276,7 +278,7 @@ void show_hint_dialog()
 	gmessage = g_strdup_printf (_("Move %s %s."), str1, str2);
 	free (str1);
 	free (str2);
-        g_warning(_("This game uses a deprecated hint method (case 3).\n Please file a bug at http://bugzilla.gnome.org including this message and\n the name of the game you where playing (look in the title bar if you \naren't sure)."));
+        g_warning(_("This game uses a deprecated hint method (case 3).\nPlease file a bug at http://bugzilla.gnome.org including this message and\nthe name of the game you where playing (look in the title bar if you \naren't sure)."));
 	break;
 
       case 4:
@@ -302,7 +304,7 @@ void show_hint_dialog()
 	                                   GTK_MESSAGE_INFO,
 	                                   GTK_BUTTONS_OK,
 	                                   gmessage);
-
+  gtk_dialog_set_has_separator (GTK_DIALOG (hint_dlg), FALSE);
   if (hint_dlg) {
 	  g_signal_connect (GTK_OBJECT (hint_dlg),
 			      "destroy",
@@ -316,79 +318,54 @@ void show_hint_dialog()
   g_free (gmessage);
 }
 
-GtkWidget *
-get_main_page (GtkWidget* dialog)
-{
-  GtkWidget *retval = gtk_vbox_new (FALSE, GNOME_PAD_SMALL);
-
-  gtk_widget_show_all (retval);
-  return retval;
-}
-
-GtkWidget *
-get_background_page (GtkWidget* dialog)
-{
-  GtkWidget *retval = gtk_vbox_new (FALSE, GNOME_PAD_SMALL);
-
-  gtk_widget_show_all (retval);
-  return retval;
-}
-
-static GtkObject* deck_edit = NULL;
+static GtkWidget * deck_edit = NULL;
 
 static void 
-property_apply (GtkWidget *w, int response)
+property_apply (GtkWidget *w, gpointer data)
 {
+  gtk_object_destroy (card_deck);
+  deck_options =  
+    gtk_card_deck_options_edit_get (GTK_CARD_DECK_OPTIONS_EDIT (w));
+  card_deck = gdk_card_deck_new (app->window, deck_options);
+  gconf_client_set_string (gconf_client, "/apps/aisleriot/card_options",
+                           deck_options, NULL);
 
-	gtk_widget_hide(w);
-
-	if ( response != GTK_RESPONSE_OK )
-		return;
-			
-  if (gdk_card_deck_options_edit_dirty 
-      (GDK_CARD_DECK_OPTIONS_EDIT (deck_edit))) {
-    gtk_object_destroy (card_deck);
-    deck_options =  
-      gdk_card_deck_options_edit_get (GDK_CARD_DECK_OPTIONS_EDIT (deck_edit));
-    card_deck = gdk_card_deck_new (app->window, deck_options);
-    gnome_config_set_string ("Deck/Options", deck_options);
-    refresh_screen();
-  }
+  refresh_screen();
 }
 
 void show_preferences_dialog () 
 {
   static GtkWidget* property_box = NULL;
-	static GtkWidget* notebook = NULL;
-
+  GtkWidget * frame;
+  
   if (!property_box) {
     property_box = gtk_dialog_new ();
-		notebook = gtk_notebook_new();
+    gtk_dialog_set_has_separator (GTK_DIALOG (property_box), FALSE);
+    gtk_window_set_title (GTK_WINDOW (property_box), _("AisleRiot Preferences"));
+    gtk_dialog_add_buttons(GTK_DIALOG(property_box),
+                           GTK_STOCK_CLOSE, GTK_RESPONSE_OK, NULL);
+    gtk_window_set_transient_for (GTK_WINDOW(property_box), GTK_WINDOW (app));
 
-		gtk_container_add (GTK_CONTAINER(GTK_DIALOG(property_box)->vbox),
-		                   notebook);
-		
-		gtk_dialog_add_buttons(GTK_DIALOG(property_box),
-		                       GTK_STOCK_OK,
-													 GTK_RESPONSE_OK,
-													 GTK_STOCK_CANCEL,
-													 GTK_RESPONSE_CANCEL,
-													 NULL);
-
-		gtk_window_set_transient_for (GTK_WINDOW(property_box), GTK_WINDOW (app));
-		gtk_window_set_modal(GTK_WINDOW(property_box), TRUE);
-  
-    deck_edit = gdk_card_deck_options_edit_new (GTK_NOTEBOOK (notebook));
-
-    g_signal_connect (G_OBJECT (property_box), "response",
+    frame = games_frame_new (_("Card Style"));
+    
+    deck_edit = gtk_card_deck_options_edit_new ();
+    gtk_container_add (GTK_CONTAINER (frame), deck_edit);
+    
+    gtk_box_pack_start (GTK_BOX (GTK_DIALOG (property_box)->vbox), frame,
+                        TRUE, TRUE, GNOME_PAD_SMALL);
+    
+    g_signal_connect (G_OBJECT (deck_edit), "changed",
                       GTK_SIGNAL_FUNC (property_apply), NULL);
+    
+    g_signal_connect (G_OBJECT (property_box), "response",
+                      GTK_SIGNAL_FUNC (gtk_widget_hide), NULL);
 
     g_signal_connect(G_OBJECT (property_box), "delete_event",
                      GTK_SIGNAL_FUNC(gtk_widget_hide), NULL);
 
     gtk_widget_show_all (property_box);
   }
-  gdk_card_deck_options_edit_set (GDK_CARD_DECK_OPTIONS_EDIT (deck_edit),
+  gtk_card_deck_options_edit_set (GTK_CARD_DECK_OPTIONS_EDIT (deck_edit),
                                   deck_options);
   gtk_window_present (GTK_WINDOW (property_box));
 }
@@ -414,7 +391,7 @@ option_apply (GtkWidget *w, int response)
     gh_call1(game_data->apply_options_lambda, options);
 }
 
-GtkWidget *
+static GtkWidget *
 get_option_page (GtkWidget* option_dialog)
 {
   SCM opts;
@@ -440,6 +417,7 @@ void show_rules_options_dialog ()
   
   if (!option_dialog && gh_scm2bool(options)) {
     option_dialog = gtk_dialog_new ();
+    gtk_dialog_set_has_separator (GTK_DIALOG (option_dialog), FALSE);
 		notebook = gtk_notebook_new();
 
 		gtk_container_add (GTK_CONTAINER(GTK_DIALOG(option_dialog)->vbox),
