@@ -1,6 +1,8 @@
+/* -*- mode:C; indent-tabs-mode:t; tab-width:8; c-basic-offset:2; -*- */
+
 /* gnome-stones - main.c
  *
- * Time-stamp: <2003-05-09 19:12:40 callum>
+ * Time-stamp: <2003-06-19 10:39:18 callum>
  *
  * Copyright (C) 1998, 2003 Carsten Schaar
  *
@@ -28,6 +30,7 @@
 #include <stdlib.h>
 #include <gdk/gdkkeysyms.h>
 #include <string.h>
+#include "main.h"
 #include "game.h"
 #include "cave.h"
 #include "object.h"
@@ -40,39 +43,6 @@
 #include <X11/Xlib.h>
 #include <X11/Xatom.h>
 
-/* You should leave 'USE_GNOME_CANVAS' undefined, because this game
-   currently doesn't support all features with gnome_canvas stuff
-   enabled.  */
-#undef USE_GNOME_CANVAS
-
-#undef USE_KEY_RELEASE
-
-
-#define APP_NAME "gnome-stones"
-
-
-/* Definitions */
-
-#define START_DELAY 3000
-#define END_DELAY 3000
-#define CURTAIN_DELAY 50
-#define COUNTDOWN_DELAY 20
-
-#define GAME_COLS  20
-#define GAME_ROWS 12
-#define GAME_SCROLL_MAX 3
-#define GAME_SCROLL_MIN 6
-
-
-#ifdef USE_KEY_RELEASE
-#define GAME_EVENTS (GDK_KEY_PRESS_MASK             |\
-		     GDK_KEY_RELEASE_MASK)
-;
-#else
-#define GAME_EVENTS (GDK_KEY_PRESS_MASK)
-#endif
-
-
 /*****************************************************************************/
 /* Used widgets. */
 
@@ -91,7 +61,8 @@ static gboolean       player_push       = FALSE;
 /* This message will be printed in the statusline, if no message was
    specified in a game file.  */
 
-static char *default_message= N_("GNOME-Stones (c) 1998 Carsten Schaar");
+static char *default_message= N_("This is GNOME Stones");
+static GConfClient *gconf_client;
 
 
 /****************************************************************************/
@@ -129,13 +100,6 @@ show_scores_dialog (gint pos);
 /****************************************************************************/
 /* Image stuff  */
 
-static void
-my_exit (GtkWidget *widget, gpointer client_data)
-{ 
-  sound_close(); 
-  exit (1);
-}
-
 static GdkPixbuf *
 load_image_from_path (const char *relative_name)
 {
@@ -155,7 +119,7 @@ load_image_from_path (const char *relative_name)
       char       buffer[1024];
       g_snprintf (buffer, sizeof(buffer), 
 		  _("An error occured while loading the image file \"%s\".\n"
-		    "Please make sure, that GNOME-Stones is "
+		    "Please make sure, that GNOME Stones is "
 		    "correctly installed!"), relative_name);
 					    
       widget= gtk_message_dialog_new (GTK_WINDOW(app), 
@@ -163,6 +127,7 @@ load_image_from_path (const char *relative_name)
                                       GTK_MESSAGE_ERROR, 
                                       GTK_BUTTONS_OK, 
                                       buffer);
+      gtk_dialog_set_has_separator (GTK_DIALOG (widget), FALSE);
       gtk_dialog_run(GTK_DIALOG(widget));
 
       gtk_widget_destroy(widget);
@@ -171,23 +136,66 @@ load_image_from_path (const char *relative_name)
   return image;
 }
 
+static void
+draw_text_centered (GdkDrawable *pixmap, gchar *markup, gint y)
+{
+  GdkGC *gc;
 
+  PangoLayout *layout;
+  if (pixmap)
+    {
+      gc = gstones_view->style->black_gc;
+      layout = gtk_widget_create_pango_layout (gstones_view, "");
+      pango_layout_set_alignment (layout, PANGO_ALIGN_CENTER);
+      pango_layout_set_width (layout, GAME_COLS * STONE_SIZE * PANGO_SCALE);
+      pango_layout_set_markup (layout, markup, -1);
+      gdk_draw_layout (pixmap, gc, 0, y, layout);
+      g_object_unref (layout);
+    }
+}
 
 static void
 title_image_load (void)
 {
-  GdkPixbuf *image= load_image_from_path ("gnome-stones/title.png");
+  gchar *markup;
+  gchar *title_text;
+  gint text_size;
+  GdkPixbuf *image;
+  GdkGC *gc;
+  GdkPixmap *tile;
 
+  image = load_image_from_path ("gnome-stones/title-tile.png");
+  if (image != NULL) 
+    {
+      gdk_pixbuf_render_pixmap_and_mask (image, &tile, NULL, 127);
+      gdk_pixbuf_unref (image);
+    }
 
-  gdk_pixbuf_render_pixmap_and_mask (image, &title_template, NULL, 127);
-  gdk_pixbuf_render_pixmap_and_mask (image, &title_image, NULL, 127);
+  title_template = gdk_pixmap_new (gstones_view->window,
+                                   GAME_COLS * STONE_SIZE,
+                                   GAME_ROWS * STONE_SIZE, -1);
 
-  gdk_pixbuf_unref (image);
+  gc = gdk_gc_new (gstones_view->window);
+  if (tile != NULL)
+    {
+      gdk_gc_set_tile (gc, tile);
+      gdk_gc_set_fill (gc, GDK_TILED);
+      gdk_draw_rectangle (title_template, gc,
+                          TRUE, 0, 0, -1, -1);
+    }
 
-  /* Redraw the game widget.  */
-  gtk_widget_draw (gstones_view, NULL);
-  
-  
+  title_text = g_strdup (_("GNOME Stones"));
+  text_size = GAME_COLS * STONE_SIZE / (strlen (title_text)+1) * PANGO_SCALE;
+
+  markup = g_strdup_printf ("<span size=\"%d\" weight=\"bold\" foreground=\"black\">%s</span>", 
+                            text_size, title_text);
+  g_free (title_text);
+  draw_text_centered (title_template, markup, 128);
+  g_free (markup);
+
+  title_image = gdk_pixmap_new (gstones_view->window,
+                                GAME_COLS * STONE_SIZE,
+                                GAME_ROWS * STONE_SIZE, -1);
 }
 
 
@@ -255,6 +263,8 @@ game_update_image (GtkWidget *widget)
 static void
 game_update_title (void)
 {
+  GdkGC *gc;
+
   /* Replace image with template.  */
   gdk_draw_drawable (title_image,
                      gstones_view->style->black_gc, title_template,
@@ -263,60 +273,64 @@ game_update_title (void)
 
   if (game)
     {
-      static char *gametitle= N_("Game title:");
-      static char *cavename = N_("Start cave:");
+      static char *gametitle = N_("Game title");
+      static char *cavename = N_("Start cave");
 
-      guint    width, height;
+      guint width, height;
       PangoLayout *layout;
       PangoFontDescription *pfd;
-      GList   *tmp= g_list_nth (game->start_caves, start_cave);
+      GList *tmp = g_list_nth (game->start_caves, start_cave);
 
-      layout = gtk_widget_create_pango_layout(gstones_view, _(gametitle));
-      pfd = pango_font_description_from_string("fixed");
-      pango_layout_set_font_description( layout, pfd);
+      layout = gtk_widget_create_pango_layout (gstones_view, _(gametitle));
+      pfd = pango_font_description_from_string ("[sans-serif][bold][16]");
+      pango_layout_set_font_description (layout, pfd);
       
       pango_layout_get_pixel_size (layout, &width, &height);
 
-      gdk_draw_layout (title_image,
-		       gstones_view->style->black_gc,
-		       30, 
+#ifdef USE_ORIG_TITLE_SCREEN
+      gc = gstones_view->style->black_gc;
+#else
+      gc = gstones_view->style->black_gc;
+#endif
+
+      gdk_draw_layout (title_image, gc,
+                       50, 
                        GAME_ROWS*STONE_SIZE-30-height*3-height/2,
                        layout);
 
-      pango_layout_set_text( layout, 
+      pango_layout_set_text (layout,
                              game->title,
                              strlen (_(game->title)));
 
-      gdk_draw_layout (title_image,
-		       gstones_view->style->black_gc,
-		       30, 
+      gdk_draw_layout (title_image, gc,
+		       80, 
                        GAME_ROWS*STONE_SIZE-30-height*2-height/2,
                        layout);
 
-      pango_layout_set_text( layout, 
+      pango_layout_set_text (layout, 
                              _(cavename),
                              strlen (_(cavename)));
 
-      gdk_draw_layout (title_image,
-		       gstones_view->style->black_gc,
-		       30, 
+      gdk_draw_layout (title_image, gc,
+		       50, 
                        GAME_ROWS*STONE_SIZE-30-height,
                        layout);
 
-      pango_layout_set_text( layout, 
+      pango_layout_set_text (layout, 
                              tmp->data,
                              strlen (tmp->data));
 
-      gdk_draw_layout (title_image,
-		       gstones_view->style->black_gc,
-		       30, 
+      gdk_draw_layout (title_image, gc,
+		       80, 
                        GAME_ROWS*STONE_SIZE-30,
                        layout);
 
     }  
 
   /* Redraw the game widget.  */
-  gtk_widget_draw (gstones_view, NULL);
+  gtk_widget_queue_draw_area (gstones_view, 0, 0,
+                              GAME_COLS * STONE_SIZE,
+                              GAME_ROWS * STONE_SIZE);
 }
 
 
@@ -475,21 +489,23 @@ game_widget_fill (void)
 /****************************************************************************/
 /* High score enabling/disabling */
 
-void update_score_state ()
+static void 
+update_score_state (void)
 {
         gchar **names = NULL;
         gfloat *scores = NULL;
         time_t *scoretimes = NULL;
 	gint top;
 
-	top = gnome_score_get_notable(APP_NAME, NULL, &names, &scores, &scoretimes);
+	top = gnome_score_get_notable (APP_NAME, NULL, &names,
+                                       &scores, &scoretimes);
 	if (top > 0) {
-		gtk_widget_set_sensitive (game_menu[5].widget, TRUE);
+		gtk_widget_set_sensitive (game_menu[6].widget, TRUE);
 		g_strfreev(names);
 		g_free(scores);
 		g_free(scoretimes);
 	} else {
-		gtk_widget_set_sensitive (game_menu[5].widget, FALSE);
+		gtk_widget_set_sensitive (game_menu[6].widget, FALSE);
 	}
 }
 
@@ -512,7 +528,7 @@ start_cave_delay_timeout (gpointer data);
 static void
 iteration_start (GStonesCave *cave);
 
-void
+static void
 curtain_ready (ViewCurtainMode mode)
 {
   switch (mode)
@@ -577,7 +593,7 @@ curtain_ready (ViewCurtainMode mode)
    after the sequence.  Setting 'cave' to a non NULL value, lets the
    program load a new cave and start it.  */
 
-void
+static void
 curtain_start (GStonesCave *cave)
 {
   curtain_cave= cave;
@@ -671,7 +687,7 @@ countdown_timeout_function (gpointer data)
    after the countdown.  Setting 'cavename' to a non NULL value, lets
    the program load a new cave and start it.  */
 
-void
+static void
 countdown_start (GStonesCave *cave)
 {
   /* When showing the countdown, there is no iteration in the
@@ -857,7 +873,9 @@ iteration_timeout_function (gpointer data)
 #ifdef USE_GNOME_CANVAS  
   game_update_image (gstones_view);
 #else
-  gtk_widget_draw (gstones_view, NULL);
+  gtk_widget_queue_draw_area (gstones_view, 0, 0,
+                              GAME_COLS * STONE_SIZE,
+                              GAME_ROWS * STONE_SIZE);
 #endif
   flags= cave->flags;
 
@@ -884,25 +902,26 @@ iteration_start (GStonesCave *newcave)
 /****************************************************************************/
 /* Additional game stuff.  */
 
-
-void
+gboolean
 load_game (const gchar *filename, guint _start_cave)
 {
-
-  char         buffer[256];
+  gchar *basename;
+  char buffer[256];
   GStonesGame *newgame;
 
-  g_return_if_fail (filename);
+  if (!filename)
+    return FALSE;
+
+  /* Zero length strings cause interesting problems later, so catch them now. */
+  if (*filename == '\0')
+    return FALSE;
   
+  basename = g_path_get_basename (filename);
+
   /* We don't need to load a game twice.  */
   if (game && (strcmp (game->filename, filename) == 0))
-    {
-      /* Give feedback in statusbar.  */
-      sprintf (buffer, "Game '%s' already loaded.", filename);
-      gnome_app_flash (GNOME_APP (app), buffer);
-      return;
-    }
-  
+    return TRUE;
+
   /* Load game.  */
   newgame= gstones_game_load (filename);
 
@@ -937,10 +956,14 @@ load_game (const gchar *filename, guint _start_cave)
       game_update_title ();
       
       /* Give feedback in statusbar.  */
-      sprintf (buffer, "Game '%s' loaded.", filename);
-      gnome_app_flash (GNOME_APP (app), buffer);
-    }
+      sprintf (buffer, "Game '%s' loaded.", basename);
 
+      gnome_app_flash (GNOME_APP (app), buffer);
+    } else
+      return FALSE;
+  
+  g_free (basename);
+  return TRUE;
 }
 
 
@@ -949,7 +972,7 @@ show_scores_dialog (gint pos)
 {
   GtkWidget *dialog;
 
-  dialog = gnome_scores_display ("GNOME-Stones", APP_NAME, NULL, pos);
+  dialog = gnome_scores_display (_("GNOME Stones"), APP_NAME, NULL, pos);
   if (dialog != NULL) {
     gtk_window_set_transient_for (GTK_WINDOW(dialog), GTK_WINDOW(app));
     gtk_window_set_modal (GTK_WINDOW(dialog), TRUE);
@@ -1006,6 +1029,11 @@ game_pause_cb (GtkWidget *widget, gpointer data)
     }
 }
 
+static void
+level_restart_cb (GtkWidget * widget, gpointer date)
+{
+  cave_player_die (cave);
+}
 
 static void
 preferences_cb (GtkWidget *widget, gpointer data)
@@ -1038,7 +1066,11 @@ about_cb (GtkWidget *widget, gpointer data)
   GdkPixbuf *pixbuf = NULL;
   
   const gchar *authors[]= {
+    N_("Main game:"),
     "Carsten Schaar <nhadcasc@fs-maphy.uni-hannover.de>",
+    "",
+    N_("Sokoban levels:"),
+    "Herman Mansilla",
     NULL
   };
   gchar *documenters[] = {
@@ -1063,9 +1095,9 @@ about_cb (GtkWidget *widget, gpointer data)
 		  g_free (filename);
 	  }
   }
-  about= gnome_about_new (_("GNOME-STONES"), VERSION,
-			  "(C) 1998 Carsten Schaar",
-			  _("A game."),
+  about= gnome_about_new (_("GNOME Stones"), VERSION,
+			  "Copyright \xc2\xa9 1998-2003 Carsten Schaar",
+			  _("Mine through dirt and collect gems."),
 			  (const char **)authors,
                           (const char **)documenters,
                           strcmp (translator_credits, "translator_credits") != 0 ? translator_credits : NULL,
@@ -1088,6 +1120,7 @@ about_cb (GtkWidget *widget, gpointer data)
 static GnomeUIInfo game_menu[]= {
   GNOMEUIINFO_MENU_NEW_GAME_ITEM(game_start_cb, NULL),
   GNOMEUIINFO_MENU_END_GAME_ITEM(game_stop_cb, NULL),
+  GNOMEUIINFO_ITEM_STOCK(N_("_Restart level"), N_("Restart the current level"), level_restart_cb, GTK_STOCK_REFRESH),
   GNOMEUIINFO_SEPARATOR,
   GNOMEUIINFO_MENU_PAUSE_GAME_ITEM (game_pause_cb, NULL),
   GNOMEUIINFO_SEPARATOR,
@@ -1257,6 +1290,13 @@ gstones_exit (GnomeClient *client, gpointer client_data)
 /*****************************************************************************/
 /* Main function.  */
 
+GConfClient *
+get_gconf_client ()
+{
+  if (!gconf_client) 
+    gconf_client = gconf_client_get_default ();
+  return gconf_client;
+}
 
 int
 main (int argc, char *argv[])
@@ -1278,10 +1318,16 @@ main (int argc, char *argv[])
        		      GNOME_PARAM_POPT_TABLE, options,
        		      GNOME_PARAM_APP_DATADIR, DATADIR,
 		      GNOME_PARAM_APP_LIBDIR, LIBDIR, NULL);
+
+  gconf_init (argc, argv, NULL);
+  gconf_client = get_gconf_client ();
+  gconf_client_add_dir (gconf_client, "/apps/gnome-stones",
+                        GCONF_CLIENT_PRELOAD_NONE, NULL);
+
   gnome_window_icon_set_default_from_file (GNOME_ICONDIR"/gnome-stones.png");
   /* That's what a gnome application needs:  */
-  app= gnome_app_new ("gnome-stones", _("GNOME-Stones"));
-  gtk_window_set_policy  (GTK_WINDOW (app), FALSE, FALSE, TRUE);
+  app = gnome_app_new ("gnome-stones", _("GNOME Stones"));
+  gtk_window_set_resizable  (GTK_WINDOW (app), FALSE);
 
   /* ... a menu line, ... */
   gnome_app_create_menus (GNOME_APP (app), main_menu);
@@ -1332,8 +1378,6 @@ main (int argc, char *argv[])
 
   joystick_set_widget (app);
 
-  gtk_widget_realize (app);
-
   gtk_widget_show (app);
 
   title_image_load ();
@@ -1346,7 +1390,8 @@ main (int argc, char *argv[])
 
   sound_init ();
 
-  play_title_music();
+  if (pref_get_sound_enabled ())
+    play_title_music();
 
   gtk_main ();
 
